@@ -9,8 +9,8 @@ namespace newsboat {
 
 class DialogsFormAction : public ListFormAction {
 public:
-	DialogsFormAction(View*, std::string formstr, ConfigContainer* cfg, RegexManager& r);
-	~DialogsFormAction() override;
+	DialogsFormAction(View&, std::string formstr, ConfigContainer* cfg, RegexManager& r);
+	~DialogsFormAction() override = default;
 	void prepare() override;
 	void init() override;
 	const std::vector<KeyMapHintEntry>& get_keymap_hint() const override;
@@ -29,8 +29,8 @@ protected:
 
 private:
 	bool process_operation(Operation op,
-		bool automatic = false,
-		std::vector<std::string>* args = nullptr) override;
+		const std::vector<std::string>& args,
+		BindingType bindingType = BindingType::BindKey) override;
 	void update_heading();
 };
 
