@@ -14,8 +14,8 @@ namespace newsboat {
 
 class DirBrowserFormAction : public FormAction {
 public:
-	DirBrowserFormAction(View*, std::string formstr, ConfigContainer* cfg);
-	~DirBrowserFormAction() override;
+	DirBrowserFormAction(View&, std::string formstr, ConfigContainer* cfg);
+	~DirBrowserFormAction() override = default;
 	void prepare() override;
 	void init() override;
 	const std::vector<KeyMapHintEntry>& get_keymap_hint() const override;
@@ -34,8 +34,8 @@ protected:
 
 private:
 	bool process_operation(Operation op,
-		bool automatic = false,
-		std::vector<std::string>* args = nullptr) override;
+		const std::vector<std::string>& args,
+		BindingType bindingType = BindingType::BindKey) override;
 	void update_title(const std::string& working_directory);
 
 	void add_directory(std::vector<file_system::FileSystemEntry>& id_at_position,
@@ -45,9 +45,12 @@ private:
 
 	std::string get_formatted_dirname(std::string dirname, mode_t mode);
 
+	LineView file_prompt_line;
 	std::string cwd;
 
 	ListWidget files_list;
+
+	View& view;
 };
 
 } // namespace newsboat
