@@ -9,13 +9,15 @@
 #include "3rd-party/json.hpp"
 #include "utils.h"
 
+class CurlHandle;
+
 using HTTPMethod = newsboat::utils::HTTPMethod;
 
 namespace newsboat {
 
 class FreshRssApi : public RemoteApi {
 public:
-	explicit FreshRssApi(ConfigContainer* c);
+	explicit FreshRssApi(ConfigContainer& c);
 	~FreshRssApi() override = default;
 	bool authenticate() override;
 	std::vector<TaggedFeedUrl> get_subscribed_urls() override;
@@ -25,7 +27,8 @@ public:
 	bool update_article_flags(const std::string& oldflags,
 		const std::string& newflags,
 		const std::string& guid) override;
-	rsspp::Feed fetch_feed(const std::string& id, CURL* cached_handle);
+	rsspp::Feed fetch_feed(const std::string& id);
+	rsspp::Feed fetch_feed(const std::string& id, CurlHandle& cached_handle);
 
 private:
 	std::vector<std::string> get_tags(xmlNode* node);
