@@ -2,6 +2,7 @@
 #define PODBOAT_VIEW_H_
 
 #include "colormanager.h"
+#include "lineview.h"
 #include "listwidget.h"
 #include "textviewwidget.h"
 #include "stflpp.h"
@@ -44,6 +45,10 @@ private:
 	PbController& ctrl;
 	newsboat::Stfl::Form dllist_form;
 	newsboat::Stfl::Form help_form;
+	LineView title_line_dllist_form;
+	LineView title_line_help_form;
+	LineView msg_line_dllist_form;
+	LineView msg_line_help_form;
 	newsboat::KeyMap& keys;
 	const newsboat::ColorManager& colorman;
 
