@@ -9,11 +9,11 @@ namespace newsboat {
 
 class UrlViewFormAction : public FormAction {
 public:
-	UrlViewFormAction(View*,
+	UrlViewFormAction(View&,
 		std::shared_ptr<RssFeed>& feed,
 		std::string formstr,
 		ConfigContainer* cfg);
-	~UrlViewFormAction() override;
+	~UrlViewFormAction() override = default;
 	void prepare() override;
 	void init() override;
 	const std::vector<KeyMapHintEntry>& get_keymap_hint() const override;
@@ -36,8 +36,8 @@ protected:
 
 private:
 	bool process_operation(Operation op,
-		bool automatic = false,
-		std::vector<std::string>* args = nullptr) override;
+		const std::vector<std::string>& args,
+		BindingType bindingType = BindingType::BindKey) override;
 	void open_current_position_in_browser(bool interactive);
 	void update_heading();
 
