@@ -84,21 +84,12 @@ mod bridged {
 
         fn podcast_mime_to_link_type(mime_type: &str, result: &mut i64) -> bool;
 
-        fn run_program(argv: &[&str], input: &str) -> String;
+        fn run_program(argv: &[&str], input: String) -> String;
 
         fn translit(tocode: &str, fromcode: &str) -> String;
         fn utf8_to_locale(text: &str) -> Vec<u8>;
         fn locale_to_utf8(text: &[u8]) -> String;
     }
-
-    extern "C++" {
-        // cxx uses `std::out_of_range`, but doesn't include the header that defines that
-        // exception. So we do it for them.
-        include!("stdexcept");
-        // Also inject a header that defines ptrdiff_t. Note this is *not* a C++ header, because
-        // cxx uses a non-C++ name of the type.
-        include!("stddef.h");
-    }
 }
 
 fn get_auth_method(method: &str) -> u64 {
