@@ -9,6 +9,8 @@
 #include <string>
 #include <sys/stat.h>
 
+#include "3rd-party/optional.hpp"
+
 #include "config.h"
 #include "controller.h"
 #include "dbexception.h"
@@ -25,7 +27,7 @@
 
 namespace newsboat {
 
-ItemListFormAction::ItemListFormAction(View* vv,
+ItemListFormAction::ItemListFormAction(View& vv,
 	std::string formstr,
 	Cache* cc,
 	FilterContainer& f,
@@ -47,11 +49,9 @@ ItemListFormAction::ItemListFormAction(View* vv,
 	register_format_styles();
 }
 
-ItemListFormAction::~ItemListFormAction() {}
-
 bool ItemListFormAction::process_operation(Operation op,
-	bool automatic,
-	std::vector<std::string>* args)
+	const std::vector<std::string>& args,
+	BindingType bindingType)
 {
 	bool quit = false;
 	bool hardquit = false;
@@ -71,11 +71,11 @@ bool ItemListFormAction::process_operation(Operation op,
 			// no need to mark item as read, the itemview already do
 			// that
 			old_itempos = itempos;
-			v->push_itemview(feed,
+			v.push_itemview(feed,
 				visible_items[itempos].first->guid());
 			invalidate(itempos);
 		} else {
-			v->get_statusline().show_error(
+			v.get_statusline().show_error(
 				_("No item selected!")); // should not happen
 		}
 	}
@@ -84,7 +84,7 @@ bool ItemListFormAction::process_operation(Operation op,
 		ScopeMeasure m1("OP_DELETE");
 		if (!visible_items.empty()) {
 			// mark as read
-			v->get_ctrl()->mark_article_read(
+			v.get_ctrl()->mark_article_read(
 				visible_items[itempos].first->guid(), true);
 			visible_items[itempos].first->set_unread(false);
 			// mark as deleted
@@ -98,14 +98,14 @@ bool ItemListFormAction::process_operation(Operation op,
 			}
 			invalidate(itempos);
 		} else {
-			v->get_statusline().show_error(
+			v.get_statusline().show_error(
 				_("No item selected!")); // should not happen
 		}
 	}
 	break;
 	case OP_DELETE_ALL: {
 		if (!cfg->get_configvalue_as_bool("confirm-delete-all-articles") ||
-			v->confirm(_("Do you really want to delete all articles (y:Yes n:No)? "),
+			v.confirm(_("Do you really want to delete all articles (y:Yes n:No)? "),
 				_("yn")) == *_("y")) {
 			ScopeMeasure m1("OP_DELETE_ALL");
 
@@ -114,7 +114,7 @@ bool ItemListFormAction::process_operation(Operation op,
 				const auto item = pair.first;
 				item_guids.push_back(item->guid());
 			}
-			v->get_ctrl()->mark_all_read(item_guids);
+			v.get_ctrl()->mark_all_read(item_guids);
 
 			for (const auto& pair : visible_items) {
 				const auto item = pair.first;
@@ -144,9 +144,10 @@ bool ItemListFormAction::process_operation(Operation op,
 
 		auto item = visible_items[itempos].first;
 		item->set_unread(false);
-		v->get_ctrl()->mark_article_read(item->guid(), true);
+		v.get_ctrl()->mark_article_read(item->guid(), true);
 		if (cfg->get_configvalue_as_bool("openbrowser-and-mark-jumps-to-next-unread")) {
-			process_operation(OP_NEXTUNREAD);
+			std::vector<std::string> args;
+			process_operation(OP_NEXTUNREAD, args);
 		} else {
 			if (itempos < visible_items.size() - 1) {
 				list.set_position(itempos + 1);
@@ -179,10 +180,10 @@ bool ItemListFormAction::process_operation(Operation op,
 			exit_code = open_unread_items_in_browser(feed, false);
 
 			if (!exit_code.has_value()) {
-				v->get_statusline().show_error(_("Failed to spawn browser"));
+				v.get_statusline().show_error(_("Failed to spawn browser"));
 				return false;
 			} else if (*exit_code != 0) {
-				v->get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
+				v.get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
 						*exit_code));
 				return false;
 			}
@@ -202,10 +203,10 @@ bool ItemListFormAction::process_operation(Operation op,
 			exit_code = open_unread_items_in_browser(feed, true);
 
 			if (!exit_code.has_value()) {
-				v->get_statusline().show_error(_("Failed to spawn browser"));
+				v.get_statusline().show_error(_("Failed to spawn browser"));
 				return false;
 			} else if (*exit_code != 0) {
-				v->get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
+				v.get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
 						*exit_code));
 				return false;
 			}
@@ -217,22 +218,22 @@ bool ItemListFormAction::process_operation(Operation op,
 		LOG(Level::INFO, "ItemListFormAction: toggling item read at pos `%u'", itempos);
 		if (!visible_items.empty()) {
 			try {
-				const auto message_lifetime = v->get_statusline().show_message_until_finished(
+				const auto message_lifetime = v.get_statusline().show_message_until_finished(
 						_("Toggling read flag for article..."));
-				if (automatic && args->size() > 0) {
-					if ((*args)[0] == "read") {
+				if (args.size() > 0) {
+					if (args.front() == "read") {
 						visible_items[itempos]
 						.first->set_unread(
 							false);
-						v->get_ctrl()->mark_article_read(
+						v.get_ctrl()->mark_article_read(
 							visible_items[itempos]
 							.first->guid(),
 							true);
-					} else if ((*args)[0] == "unread") {
+					} else if (args.front() == "unread") {
 						visible_items[itempos]
 						.first->set_unread(
 							true);
-						v->get_ctrl()->mark_article_read(
+						v.get_ctrl()->mark_article_read(
 							visible_items[itempos]
 							.first->guid(),
 							false);
@@ -250,13 +251,13 @@ bool ItemListFormAction::process_operation(Operation op,
 						.first->unread();
 					visible_items[itempos]
 					.first->set_unread(!unread);
-					v->get_ctrl()->mark_article_read(
+					v.get_ctrl()->mark_article_read(
 						visible_items[itempos]
 						.first->guid(),
 						unread);
 				}
 			} catch (const DbException& e) {
-				v->get_statusline().show_error(strprintf::fmt(
+				v.get_statusline().show_error(strprintf::fmt(
 						_("Error while toggling read flag: %s"),
 						e.what()));
 			}
@@ -266,7 +267,8 @@ bool ItemListFormAction::process_operation(Operation op,
 					list.set_position(itempos + 1);
 				}
 			} else {
-				process_operation(OP_NEXTUNREAD);
+				std::vector<std::string> args;
+				process_operation(OP_NEXTUNREAD, args);
 			}
 			invalidate(itempos);
 		}
@@ -298,9 +300,9 @@ bool ItemListFormAction::process_operation(Operation op,
 						links,
 						baseurl);
 					if (!links.empty()) {
-						v->push_urlview(links, feed);
+						v.push_urlview(links, feed);
 					} else {
-						v->get_statusline().show_error(
+						v.get_statusline().show_error(
 							_("URL list empty."));
 					}
 				} else {
@@ -310,7 +312,7 @@ bool ItemListFormAction::process_operation(Operation op,
 				}
 			}
 		} else {
-			v->get_statusline().show_error(
+			v.get_statusline().show_error(
 				_("No item selected!")); // should not happen
 		}
 		break;
@@ -318,27 +320,46 @@ bool ItemListFormAction::process_operation(Operation op,
 		LOG(Level::INFO, "ItemListFormAction: bookmarking item at pos `%u'", itempos);
 		if (!visible_items.empty()) {
 			if (itempos < visible_items.size()) {
-				if (automatic) {
+				switch (bindingType) {
+				case BindingType::Bind:
+					if (args.empty()) {
+						this->start_bookmark_qna(
+							visible_items[itempos].first->title(),
+							visible_items[itempos].first->link(),
+							feed->title());
+					} else {
+						qna_responses = {
+							visible_items[itempos].first->link(),
+							utils::utf8_to_locale(visible_items[itempos].first->title()),
+							args.front(),
+							feed->title(),
+						};
+						this->finished_qna(OP_INT_BM_END);
+					}
+					break;
+				case BindingType::Macro:
 					qna_responses.clear();
 					qna_responses.push_back(
 						visible_items[itempos]
 						.first->link());
 					qna_responses.push_back(utils::utf8_to_locale(
 							visible_items[itempos].first->title()));
-					qna_responses.push_back(args->size() > 0
-						? (*args)[0]
+					qna_responses.push_back(args.size() > 0
+						? args.front()
 						: "");
 					qna_responses.push_back(feed->title());
 					this->finished_qna(OP_INT_BM_END);
-				} else {
+					break;
+				case BindingType::BindKey:
 					this->start_bookmark_qna(
 						visible_items[itempos].first->title(),
 						visible_items[itempos].first->link(),
 						feed->title());
+					break;
 				}
 			}
 		} else {
-			v->get_statusline().show_error(
+			v.get_statusline().show_error(
 				_("No item selected!")); // should not happen
 		}
 	}
@@ -346,25 +367,37 @@ bool ItemListFormAction::process_operation(Operation op,
 	case OP_EDITFLAGS: {
 		if (!visible_items.empty()) {
 			if (itempos < visible_items.size()) {
-				if (automatic) {
-					if (args->size() > 0) {
+				switch (bindingType) {
+				case BindingType::Bind:
+					if (args.empty()) {
+						std::vector<QnaPair> qna {
+							QnaPair(_("Flags: "), visible_items[itempos].first->flags())
+						};
+						this->start_qna(qna, OP_INT_EDITFLAGS_END);
+					} else {
+						qna_responses = {args.front()};
+						finished_qna(OP_INT_EDITFLAGS_END);
+					}
+					break;
+				case BindingType::Macro:
+					if (args.size() > 0) {
 						qna_responses.clear();
 						qna_responses.push_back(
-							(*args)[0]);
+							args.front());
 						finished_qna(
 							OP_INT_EDITFLAGS_END);
 					}
-				} else {
+					break;
+				case BindingType::BindKey:
 					std::vector<QnaPair> qna;
 					qna.push_back(QnaPair(_("Flags: "),
-							visible_items[itempos]
-							.first->flags()));
-					this->start_qna(
-						qna, OP_INT_EDITFLAGS_END);
+							visible_items[itempos].first->flags()));
+					this->start_qna(qna, OP_INT_EDITFLAGS_END);
+					break;
 				}
 			}
 		} else {
-			v->get_statusline().show_error(
+			v.get_statusline().show_error(
 				_("No item selected!")); // should not happen
 		}
 	}
@@ -373,19 +406,31 @@ bool ItemListFormAction::process_operation(Operation op,
 		LOG(Level::INFO, "ItemListFormAction: saving item at pos `%u'", itempos);
 		if (!visible_items.empty()) {
 			std::shared_ptr<RssItem> item = visible_items[itempos].first;
-			std::string filename;
-			if (automatic) {
-				if (args->size() > 0) {
-					filename = (*args)[0];
+			nonstd::optional<std::string> filename;
+			switch (bindingType) {
+			case BindingType::Bind:
+				if (args.empty()) {
+					const auto title = utils::utf8_to_locale(item->title());
+					const auto suggestion = v.get_filename_suggestion(title);
+					filename = v.run_filebrowser(suggestion);
+				} else {
+					filename = args.front();
 				}
-			} else {
+				break;
+			case BindingType::Macro:
+				if (args.size() > 0) {
+					filename = args.front();
+				}
+				break;
+			case BindingType::BindKey:
 				const auto title = utils::utf8_to_locale(item->title());
-				const auto suggestion = v->get_filename_suggestion(title);
-				filename = v->run_filebrowser(suggestion);
+				const auto suggestion = v.get_filename_suggestion(title);
+				filename = v.run_filebrowser(suggestion);
+				break;
 			}
 			save_article(filename, item);
 		} else {
-			v->get_statusline().show_error(_("Error: no item selected!"));
+			v.get_statusline().show_error(_("Error: no item selected!"));
 		}
 	}
 	break;
@@ -393,23 +438,23 @@ bool ItemListFormAction::process_operation(Operation op,
 		handle_op_saveall();
 		break;
 	case OP_HELP:
-		v->push_help();
+		v.push_help();
 		break;
 	case OP_RELOAD:
 		LOG(Level::INFO, "ItemListFormAction: reloading current feed");
-		v->get_ctrl()->get_reloader()->reload(pos);
+		v.get_ctrl()->get_reloader()->reload(pos);
 		invalidate_list();
 		break;
 	case OP_QUIT:
 		LOG(Level::INFO, "ItemListFormAction: quitting");
-		v->feedlist_mark_pos_if_visible(pos);
+		v.feedlist_mark_pos_if_visible(pos);
 		feed->purge_deleted_items();
 		feed->unload();
 		quit = true;
 		break;
 	case OP_HARDQUIT:
 		LOG(Level::INFO, "ItemListFormAction: hard quitting");
-		v->feedlist_mark_pos_if_visible(pos);
+		v.feedlist_mark_pos_if_visible(pos);
 		feed->purge_deleted_items();
 		hardquit = true;
 		break;
@@ -417,8 +462,8 @@ bool ItemListFormAction::process_operation(Operation op,
 		LOG(Level::INFO,
 			"ItemListFormAction: jumping to next unread item");
 		if (!jump_to_next_unread_item(false)) {
-			if (!v->get_next_unread(*this)) {
-				v->get_statusline().show_error(_("No unread items."));
+			if (!v.get_next_unread(*this)) {
+				v.get_statusline().show_error(_("No unread items."));
 			}
 		}
 		break;
@@ -426,16 +471,16 @@ bool ItemListFormAction::process_operation(Operation op,
 		LOG(Level::INFO,
 			"ItemListFormAction: jumping to previous unread item");
 		if (!jump_to_previous_unread_item(false)) {
-			if (!v->get_previous_unread(*this)) {
-				v->get_statusline().show_error(_("No unread items."));
+			if (!v.get_previous_unread(*this)) {
+				v.get_statusline().show_error(_("No unread items."));
 			}
 		}
 		break;
 	case OP_NEXT:
 		LOG(Level::INFO, "ItemListFormAction: jumping to next item");
 		if (!jump_to_next_item(false)) {
-			if (!v->get_next(*this)) {
-				v->get_statusline().show_error(_("Already on last item."));
+			if (!v.get_next(*this)) {
+				v.get_statusline().show_error(_("Already on last item."));
 			}
 		}
 		break;
@@ -443,46 +488,46 @@ bool ItemListFormAction::process_operation(Operation op,
 		LOG(Level::INFO,
 			"ItemListFormAction: jumping to previous item");
 		if (!jump_to_previous_item(false)) {
-			if (!v->get_previous(*this)) {
-				v->get_statusline().show_error(_("Already on first item."));
+			if (!v.get_previous(*this)) {
+				v.get_statusline().show_error(_("Already on first item."));
 			}
 		}
 		break;
 	case OP_RANDOMUNREAD:
 		if (!jump_to_random_unread_item()) {
-			if (!v->get_random_unread(*this)) {
-				v->get_statusline().show_error(_("No unread items."));
+			if (!v.get_random_unread(*this)) {
+				v.get_statusline().show_error(_("No unread items."));
 			}
 		}
 		break;
 	case OP_NEXTUNREADFEED:
-		if (!v->get_next_unread_feed(*this)) {
-			v->get_statusline().show_error(_("No unread feeds."));
+		if (!v.get_next_unread_feed(*this)) {
+			v.get_statusline().show_error(_("No unread feeds."));
 		}
 		break;
 	case OP_PREVUNREADFEED:
-		if (!v->get_prev_unread_feed(*this)) {
-			v->get_statusline().show_error(_("No unread feeds."));
+		if (!v.get_prev_unread_feed(*this)) {
+			v.get_statusline().show_error(_("No unread feeds."));
 		}
 		break;
 	case OP_NEXTFEED:
-		if (!v->get_next_feed(*this)) {
-			v->get_statusline().show_error(_("Already on last feed."));
+		if (!v.get_next_feed(*this)) {
+			v.get_statusline().show_error(_("Already on last feed."));
 		}
 		break;
 	case OP_PREVFEED:
-		if (!v->get_prev_feed(*this)) {
-			v->get_statusline().show_error(_("Already on first feed."));
+		if (!v.get_prev_feed(*this)) {
+			v.get_statusline().show_error(_("Already on first feed."));
 		}
 		break;
 	case OP_MARKFEEDREAD:
 		if (!cfg->get_configvalue_as_bool(
 				"confirm-mark-feed-read") ||
-			v->confirm(_("Do you really want to mark this feed as read (y:Yes n:No)? "),
+			v.confirm(_("Do you really want to mark this feed as read (y:Yes n:No)? "),
 				_("yn")) == *_("y")) {
 			LOG(Level::INFO, "ItemListFormAction: marking feed read");
 			try {
-				const auto message_lifetime = v->get_statusline().show_message_until_finished(
+				const auto message_lifetime = v.get_statusline().show_message_until_finished(
 						_("Marking feed read..."));
 
 				std::vector<std::string> guids;
@@ -494,9 +539,9 @@ bool ItemListFormAction::process_operation(Operation op,
 
 				if (filter_active) {
 					// We're only viewing a subset of items, so mark them off one by one.
-					v->get_ctrl()->mark_all_read(guids);
+					v.get_ctrl()->mark_all_read(guids);
 				} else {
-					v->get_ctrl()->mark_all_read(pos);
+					v.get_ctrl()->mark_all_read(pos);
 				}
 
 				if (visible_items.size() > 0) {
@@ -507,7 +552,8 @@ bool ItemListFormAction::process_operation(Operation op,
 					}
 				}
 				if (cfg->get_configvalue_as_bool("markfeedread-jumps-to-next-unread")) {
-					process_operation(OP_NEXTUNREAD);
+					std::vector<std::string> args;
+					process_operation(OP_NEXTUNREAD, args);
 				} else { // reposition to first/last item
 					std::string sortorder =
 						cfg->get_configvalue("article-sort-order");
@@ -527,7 +573,7 @@ bool ItemListFormAction::process_operation(Operation op,
 				}
 				invalidate_list();
 			} catch (const DbException& e) {
-				v->get_statusline().show_error(strprintf::fmt(
+				v.get_statusline().show_error(strprintf::fmt(
 						_("Error: couldn't mark feed read: %s"),
 						e.what()));
 			}
@@ -536,14 +582,14 @@ bool ItemListFormAction::process_operation(Operation op,
 	case OP_MARKALLABOVEASREAD: {
 		LOG(Level::INFO,
 			"ItemListFormAction: marking all above as read");
-		const auto message_lifetime = v->get_statusline().show_message_until_finished(
+		const auto message_lifetime = v.get_statusline().show_message_until_finished(
 				_("Marking all above as read..."));
 		if (itempos < visible_items.size()) {
 			for (unsigned int i = 0; i < itempos; ++i) {
 				if (visible_items[i].first->unread()) {
 					visible_items[i].first->set_unread(
 						false);
-					v->get_ctrl()->mark_article_read(
+					v.get_ctrl()->mark_article_read(
 						visible_items[i].first->guid(),
 						true);
 				}
@@ -570,86 +616,138 @@ bool ItemListFormAction::process_operation(Operation op,
 	case OP_PIPE_TO:
 		if (visible_items.size() != 0) {
 			std::vector<QnaPair> qna;
-			if (automatic) {
-				if (args->size() > 0) {
-					qna_responses.clear();
-					qna_responses.push_back((*args)[0]);
+			switch (bindingType) {
+			case BindingType::Bind:
+				if (args.empty()) {
+					qna.push_back(QnaPair(_("Pipe article to command: "), ""));
+					this->start_qna(qna, OP_PIPE_TO, &cmdlinehistory);
+				} else {
+					qna_responses = { args.front() };
 					finished_qna(OP_PIPE_TO);
 				}
-			} else {
+				break;
+			case BindingType::Macro:
+				if (args.size() > 0) {
+					qna_responses.clear();
+					qna_responses.push_back(args.front());
+					finished_qna(OP_PIPE_TO);
+				}
+				break;
+			case BindingType::BindKey:
 				qna.push_back(QnaPair(
 						_("Pipe article to command: "), ""));
 				this->start_qna(
 					qna, OP_PIPE_TO, &cmdlinehistory);
+				break;
 			}
 		} else {
-			v->get_statusline().show_error(_("No item selected!"));
+			v.get_statusline().show_error(_("No item selected!"));
 		}
 		break;
 	case OP_SEARCH: {
 		std::vector<QnaPair> qna;
-		if (automatic) {
-			if (args->size() > 0) {
-				qna_responses.clear();
-				qna_responses.push_back((*args)[0]);
+		switch (bindingType) {
+		case BindingType::Bind:
+			if (args.empty()) {
+				qna.push_back(QnaPair(_("Search for: "), ""));
+				this->start_qna(qna, OP_INT_START_SEARCH, &searchhistory);
+			} else {
+				qna_responses = { args.front() };
 				finished_qna(OP_INT_START_SEARCH);
 			}
-		} else {
+			break;
+		case BindingType::Macro:
+			if (args.size() > 0) {
+				qna_responses.clear();
+				qna_responses.push_back(args.front());
+				finished_qna(OP_INT_START_SEARCH);
+			}
+			break;
+		case BindingType::BindKey:
 			qna.push_back(QnaPair(_("Search for: "), ""));
 			this->start_qna(
 				qna, OP_INT_START_SEARCH, &searchhistory);
+			break;
 		}
 	}
 	break;
 	case OP_GOTO_TITLE:
-		if (automatic) {
-			if (args->size() >= 1) {
+		switch (bindingType) {
+		case BindingType::Bind:
+			if (args.empty()) {
+				std::vector<QnaPair> qna {
+					QnaPair(_("Title: "), ""),
+				};
+				this->start_qna(qna, OP_INT_GOTO_TITLE);
+			} else {
 				qna_responses = {args[0]};
 				finished_qna(OP_INT_GOTO_TITLE);
 			}
-		} else {
+			break;
+		case BindingType::Macro:
+			if (args.size() >= 1) {
+				qna_responses = {args[0]};
+				finished_qna(OP_INT_GOTO_TITLE);
+			}
+			break;
+		case BindingType::BindKey:
 			std::vector<QnaPair> qna;
 			qna.push_back(QnaPair(_("Title: "), ""));
 			this->start_qna(qna, OP_INT_GOTO_TITLE);
+			break;
 		}
 		break;
 	case OP_EDIT_URLS:
-		v->get_ctrl()->edit_urls_file();
+		v.get_ctrl()->edit_urls_file();
 		break;
 	case OP_SELECTFILTER:
 		if (filter_container.size() > 0) {
 			std::string newfilter;
-			if (automatic && args->size() > 0) {
-				const std::string filter_name = (*args)[0];
+			if (args.size() > 0) {
+				const std::string filter_name = args.front();
 				const auto filter = filter_container.get_filter(filter_name);
 
 				if (filter.has_value()) {
 					apply_filter(filter.value());
 				} else {
-					v->get_statusline().show_error(strprintf::fmt(_("No filter found with name `%s'."),
+					v.get_statusline().show_error(strprintf::fmt(_("No filter found with name `%s'."),
 							filter_name));
 				}
 			} else {
-				const std::string filter_text = v->select_filter(filter_container.get_filters());
+				const std::string filter_text = v.select_filter(filter_container.get_filters());
 				apply_filter(filter_text);
 			}
 		} else {
-			v->get_statusline().show_error(_("No filters defined."));
+			v.get_statusline().show_error(_("No filters defined."));
 		}
 
 		break;
 	case OP_SETFILTER:
-		if (automatic) {
-			if (args->size() > 0) {
-				qna_responses.clear();
-				qna_responses.push_back((*args)[0]);
+		switch (bindingType) {
+		case BindingType::Bind:
+			if (args.empty()) {
+				std::vector<QnaPair> qna {
+					QnaPair(_("Filter: "), ""),
+				};
+				this->start_qna(qna, OP_INT_END_SETFILTER, &filterhistory);
+			} else {
+				qna_responses = { args.front() };
 				this->finished_qna(OP_INT_END_SETFILTER);
 			}
-		} else {
+			break;
+		case BindingType::Macro:
+			if (args.size() > 0) {
+				qna_responses.clear();
+				qna_responses.push_back(args.front());
+				this->finished_qna(OP_INT_END_SETFILTER);
+			}
+			break;
+		case BindingType::BindKey:
 			std::vector<QnaPair> qna;
 			qna.push_back(QnaPair(_("Filter: "), ""));
 			this->start_qna(
 				qna, OP_INT_END_SETFILTER, &filterhistory);
+			break;
 		}
 		break;
 	case OP_CLEARFILTER:
@@ -662,7 +760,7 @@ bool ItemListFormAction::process_operation(Operation op,
 		// "Sort by (d)ate/..." and "Reverse Sort by (d)ate/..."
 		// messages
 		std::string input_options = _("dtfalgr");
-		char c = v->confirm(
+		char c = v.confirm(
 				_("Sort by "
 					"(d)ate/(t)itle/(f)lags/(a)uthor/(l)ink/(g)uid/(r)andom?"),
 				input_options);
@@ -700,7 +798,7 @@ bool ItemListFormAction::process_operation(Operation op,
 	break;
 	case OP_REVSORT: {
 		std::string input_options = _("dtfalgr");
-		char c = v->confirm(
+		char c = v.confirm(
 				_("Reverse Sort by "
 					"(d)ate/(t)itle/(f)lags/(a)uthor/(l)ink/(g)uid/(r)andom?"),
 				input_options);
@@ -741,14 +839,14 @@ bool ItemListFormAction::process_operation(Operation op,
 	case OP_ENQUEUE:
 		if (!visible_items.empty() && itempos < visible_items.size()) {
 			const auto item = visible_items[itempos].first;
-			return enqueue_item_enclosure(item, feed, *v, *rsscache);
+			return enqueue_item_enclosure(item, feed, v, *rsscache);
 		} else {
-			v->get_statusline().show_error(_("No item selected!"));
+			v.get_statusline().show_error(_("No item selected!"));
 			return false;
 		}
 		break;
 	case OP_ARTICLEFEED: {
-		auto feeds = v->get_ctrl()->get_feedcontainer()->get_all_feeds();
+		auto feeds = v.get_ctrl()->get_feedcontainer()->get_all_feeds();
 		size_t pos;
 		auto article_feed = visible_items[itempos].first->get_feedptr();
 		for (pos = 0; pos < feeds.size(); pos++) {
@@ -757,20 +855,20 @@ bool ItemListFormAction::process_operation(Operation op,
 			}
 		}
 		if (pos != feeds.size()) {
-			v->push_itemlist(pos);
+			v.push_itemlist(pos);
 		}
 	}
 	break;
 	default:
-		ListFormAction::process_operation(op, automatic, args);
+		ListFormAction::process_operation(op, args, bindingType);
 		break;
 	}
 	if (hardquit) {
-		while (v->formaction_stack_size() > 0) {
-			v->pop_current_formaction();
+		while (v.formaction_stack_size() > 0) {
+			v.pop_current_formaction();
 		}
 	} else if (quit) {
-		v->pop_current_formaction();
+		v.pop_current_formaction();
 	}
 	return true;
 }
@@ -786,19 +884,19 @@ bool ItemListFormAction::open_position_in_browser(
 		const auto item = visible_items[pos].first;
 		const auto link = item->link();
 		const auto feedurl = item->feedurl();
-		const auto exit_code = v->open_in_browser(link, feedurl, "article", item->title(),
+		const auto exit_code = v.open_in_browser(link, feedurl, "article", item->title(),
 				interactive);
 		if (!exit_code.has_value()) {
-			v->get_statusline().show_error(_("Failed to spawn browser"));
+			v.get_statusline().show_error(_("Failed to spawn browser"));
 			return false;
 		} else if (*exit_code != 0) {
-			v->get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
+			v.get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
 					*exit_code));
 			return false;
 		}
 		return true;
 	} else {
-		v->get_statusline().show_error(_("No item selected!"));
+		v.get_statusline().show_error(_("No item selected!"));
 		return false;
 	}
 }
@@ -829,9 +927,9 @@ void ItemListFormAction::finished_qna(Operation op)
 			unsigned int itempos = list.get_position();
 			std::string cmd = qna_responses[0];
 			std::ostringstream ostr;
-			v->get_ctrl()->write_item(
+			v.get_ctrl()->write_item(
 				visible_items[itempos].first, ostr);
-			v->push_empty_formaction();
+			v.push_empty_formaction();
 			Stfl::reset();
 			FILE* f = popen(cmd.c_str(), "w");
 			if (f) {
@@ -839,8 +937,8 @@ void ItemListFormAction::finished_qna(Operation op)
 				fwrite(data.c_str(), data.length(), 1, f);
 				pclose(f);
 			}
-			v->drop_queued_input();
-			v->pop_current_formaction();
+			v.drop_queued_input();
+			v.pop_current_formaction();
 		}
 	}
 	break;
@@ -859,15 +957,15 @@ void ItemListFormAction::qna_end_setfilter()
 void ItemListFormAction::qna_end_editflags()
 {
 	if (visible_items.empty()) {
-		v->get_statusline().show_error(_("No item selected!")); // should not happen
+		v.get_statusline().show_error(_("No item selected!")); // should not happen
 		return;
 	}
 
 	const unsigned int itempos = list.get_position();
 	if (itempos < visible_items.size()) {
 		visible_items[itempos].first->set_flags(qna_responses[0]);
-		v->get_ctrl()->update_flags(visible_items[itempos].first);
-		v->get_statusline().show_message(_("Flags updated."));
+		v.get_ctrl()->update_flags(visible_items[itempos].first);
+		v.get_statusline().show_message(_("Flags updated."));
 		LOG(Level::DEBUG,
 			"ItemListFormAction::finished_qna: updated flags");
 		invalidate(itempos);
@@ -884,13 +982,13 @@ void ItemListFormAction::qna_start_search()
 	searchhistory.add_line(searchphrase);
 	std::vector<std::shared_ptr<RssItem>> items;
 	try {
-		const auto message_lifetime = v->get_statusline().show_message_until_finished(
+		const auto message_lifetime = v.get_statusline().show_message_until_finished(
 				_("Searching..."));
 		const auto utf8searchphrase = utils::locale_to_utf8(searchphrase);
-		items = v->get_ctrl()->search_for_items(
+		items = v.get_ctrl()->search_for_items(
 				utf8searchphrase, feed);
 	} catch (const DbException& e) {
-		v->get_statusline().show_error(
+		v.get_statusline().show_error(
 			strprintf::fmt(_("Error while searching for `%s': %s"),
 				searchphrase,
 				e.what()));
@@ -898,14 +996,14 @@ void ItemListFormAction::qna_start_search()
 	}
 
 	if (items.empty()) {
-		v->get_statusline().show_error(_("No results."));
+		v.get_statusline().show_error(_("No results."));
 		return;
 	}
 
 	std::shared_ptr<RssFeed> search_dummy_feed(new RssFeed(rsscache, ""));
 	search_dummy_feed->set_search_feed(true);
 	search_dummy_feed->add_items(items);
-	v->push_searchresult(search_dummy_feed, searchphrase);
+	v.push_searchresult(search_dummy_feed, searchphrase);
 }
 
 void ItemListFormAction::do_update_visible_items()
@@ -1002,7 +1100,7 @@ void ItemListFormAction::prepare()
 	try {
 		do_update_visible_items();
 	} catch (MatcherException& e) {
-		v->get_statusline().show_error(strprintf::fmt(
+		v.get_statusline().show_error(strprintf::fmt(
 				_("Error: applying the filter failed: %s"), e.what()));
 		return;
 	}
@@ -1012,7 +1110,7 @@ void ItemListFormAction::prepare()
 			const unsigned int itempos = list.get_position();
 			if (visible_items[itempos].first->unread()) {
 				visible_items[itempos].first->set_unread(false);
-				v->get_ctrl()->mark_article_read(
+				v.get_ctrl()->mark_article_read(
 					visible_items[itempos].first->guid(),
 					true);
 				invalidate(itempos);
@@ -1126,7 +1224,7 @@ void ItemListFormAction::init()
 {
 	recalculate_widget_dimensions();
 	list.set_position(0);
-	set_value("msg", "");
+	set_status("");
 	set_keymap_hints();
 	invalidate_list();
 	do_update_visible_items();
@@ -1171,15 +1269,12 @@ void ItemListFormAction::set_head(const std::string& s,
 	unsigned int total,
 	const std::string& url)
 {
-	std::string title;
-
 	FmtStrFormatter fmt = setup_head_formatter(s, unread, total, url);
 
 	const unsigned int width = utils::to_u(f.get("title:w"));
-	title = fmt.do_format(
+	set_title(fmt.do_format(
 			cfg->get_configvalue("articlelist-title-format"),
-			width);
-	set_value("head", title);
+			width));
 }
 
 bool ItemListFormAction::jump_to_previous_unread_item(bool start_with_last)
@@ -1315,12 +1410,12 @@ void ItemListFormAction::handle_cmdline_num(unsigned int idx)
 		idx <= visible_items[visible_items.size() - 1].second + 1) {
 		int i = get_pos(idx - 1);
 		if (i == -1) {
-			v->get_statusline().show_error(_("Position not visible!"));
+			v.get_statusline().show_error(_("Position not visible!"));
 		} else {
 			list.set_position(i);
 		}
 	} else {
-		v->get_statusline().show_error(_("Invalid position!"));
+		v.get_statusline().show_error(_("Invalid position!"));
 	}
 }
 
@@ -1370,20 +1465,20 @@ void ItemListFormAction::restore_selected_position()
 
 }
 
-void ItemListFormAction::save_article(const std::string& filename,
+void ItemListFormAction::save_article(const nonstd::optional<std::string>& filename,
 	std::shared_ptr<RssItem> item)
 {
-	if (filename == "") {
-		v->get_statusline().show_error(_("Aborted saving."));
+	if (!filename.has_value()) {
+		v.get_statusline().show_error(_("Aborted saving."));
 	} else {
 		try {
-			v->get_ctrl()->write_item(item, filename);
-			v->get_statusline().show_message(strprintf::fmt(
-					_("Saved article to %s"), filename));
+			v.get_ctrl()->write_item(item, filename.value());
+			v.get_statusline().show_message(strprintf::fmt(
+					_("Saved article to %s"), filename.value()));
 		} catch (...) {
-			v->get_statusline().show_error(strprintf::fmt(
+			v.get_statusline().show_error(strprintf::fmt(
 					_("Error: couldn't save article to %s"),
-					filename));
+					filename.value()));
 		}
 	}
 }
@@ -1391,11 +1486,11 @@ void ItemListFormAction::save_article(const std::string& filename,
 void ItemListFormAction::handle_save(const std::vector<std::string>& cmd_args)
 {
 	if (cmd_args.size() < 1) {
-		v->get_statusline().show_error(_("Error: no filename provided"));
+		v.get_statusline().show_error(_("Error: no filename provided"));
 		return;
 	}
 	if (visible_items.empty()) {
-		v->get_statusline().show_error(_("Error: no item selected!"));
+		v.get_statusline().show_error(_("Error: no item selected!"));
 		return;
 	}
 	const std::string filename = utils::resolve_tilde(cmd_args.front());
@@ -1496,19 +1591,19 @@ void ItemListFormAction::handle_op_saveall()
 		return;
 	}
 
-	std::string directory = v->run_dirbrowser();
+	nonstd::optional<std::string> directory = v.run_dirbrowser();
 
-	if (directory.empty()) {
+	if (!directory.has_value()) {
 		return;
 	}
 
-	if (directory.back() != NEWSBEUTER_PATH_SEP) {
-		directory.push_back(NEWSBEUTER_PATH_SEP);
+	if (directory.value().back() != NEWSBEUTER_PATH_SEP) {
+		directory.value().push_back(NEWSBEUTER_PATH_SEP);
 	}
 
 	std::vector<std::string> filenames;
 	for (const auto& item : visible_items) {
-		filenames.emplace_back( utils::utf8_to_locale(v->get_filename_suggestion(
+		filenames.emplace_back( utils::utf8_to_locale(v.get_filename_suggestion(
 					item.first->title())));
 	}
 
@@ -1518,7 +1613,7 @@ void ItemListFormAction::handle_op_saveall()
 
 	int nfiles_exist = filenames.size() - unique_filenames.size();
 	for (const auto& filename : unique_filenames) {
-		const auto filepath = directory + filename;
+		const auto filepath = directory.value() + filename;
 		struct stat sbuf;
 		if (::stat(filepath.c_str(), &sbuf) != -1) {
 			nfiles_exist++;
@@ -1539,7 +1634,7 @@ void ItemListFormAction::handle_op_saveall()
 	bool overwrite_all = false;
 	for (size_t item_idx = 0; item_idx < filenames.size(); ++item_idx) {
 		const auto filename = filenames[item_idx];
-		const auto filepath = directory + filename;
+		const auto filepath = directory.value() + filename;
 		auto item = visible_items[item_idx].first;
 
 		struct stat sbuf;
@@ -1551,17 +1646,17 @@ void ItemListFormAction::handle_op_saveall()
 
 			char c;
 			if (nfiles_exist > 1) {
-				c = v->confirm(strprintf::fmt(
+				c = v.confirm(strprintf::fmt(
 							_("Overwrite `%s' in `%s'? "
 								"There are %d more conflicts like this "
 								"(y:Yes a:Yes to all n:No q:No to all)"),
-							filename, directory, --nfiles_exist),
+							filename, directory.value(), --nfiles_exist),
 						input_options);
 			} else {
-				c = v->confirm(strprintf::fmt(
+				c = v.confirm(strprintf::fmt(
 							_("Overwrite `%s' in `%s'? "
 								"(y:Yes n:No)"),
-							filename, directory),
+							filename, directory.value()),
 						input_options);
 			}
 			if (!c) {
@@ -1593,7 +1688,7 @@ void ItemListFormAction::apply_filter(const std::string& filtertext)
 
 	filterhistory.add_line(filtertext);
 	if (!matcher.parse(filtertext)) {
-		v->get_statusline().show_error(strprintf::fmt(
+		v.get_statusline().show_error(strprintf::fmt(
 				_("Error: couldn't parse filter expression `%s': %s"),
 				filtertext,
 				matcher.get_parse_error()));
