@@ -6,6 +6,7 @@
 #include <vector>
 
 #include "config.h"
+#include "curldatareceiver.h"
 #include "curlhandle.h"
 #include "strprintf.h"
 #include "utils.h"
@@ -26,7 +27,7 @@
 
 namespace newsboat {
 
-OldReaderApi::OldReaderApi(ConfigContainer* c)
+OldReaderApi::OldReaderApi(ConfigContainer& c)
 	: RemoteApi(c)
 {
 }
@@ -38,14 +39,6 @@ bool OldReaderApi::authenticate()
 	return auth != "";
 }
 
-static size_t my_write_data(void* buffer, size_t size, size_t nmemb,
-	void* userp)
-{
-	std::string* pbuf = static_cast<std::string*>(userp);
-	pbuf->append(static_cast<const char*>(buffer), size * nmemb);
-	return size * nmemb;
-}
-
 std::string OldReaderApi::retrieve_auth()
 {
 	CurlHandle handle;
@@ -68,15 +61,15 @@ std::string OldReaderApi::retrieve_auth()
 	curl_free(username);
 	curl_free(password);
 
-	std::string result;
-
 	utils::set_common_curl_options(handle, cfg);
-	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEFUNCTION, my_write_data);
-	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEDATA, &result);
 	curl_easy_setopt(handle.ptr(), CURLOPT_POSTFIELDS, postcontent.c_str());
 	curl_easy_setopt(handle.ptr(), CURLOPT_URL, OLDREADER_LOGIN);
+
+	auto curlDataReceiver = CurlDataReceiver::register_data_handler(handle);
+
 	curl_easy_perform(handle.ptr());
 
+	const std::string result = curlDataReceiver->get_data();
 	std::vector<std::string> lines = utils::tokenize(result);
 	for (const auto& line : lines) {
 		LOG(Level::DEBUG,
@@ -97,17 +90,18 @@ std::vector<TaggedFeedUrl> OldReaderApi::get_subscribed_urls()
 	curl_slist* custom_headers{};
 
 	CurlHandle handle;
-	std::string result;
 	add_custom_headers(&custom_headers);
 	curl_easy_setopt(handle.ptr(), CURLOPT_HTTPHEADER, custom_headers);
 
 	utils::set_common_curl_options(handle, cfg);
-	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEFUNCTION, my_write_data);
-	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEDATA, &result);
 	curl_easy_setopt(handle.ptr(), CURLOPT_URL, OLDREADER_SUBSCRIPTION_LIST);
+
+	auto curlDataReceiver = CurlDataReceiver::register_data_handler(handle);
+
 	curl_easy_perform(handle.ptr());
 	curl_slist_free_all(custom_headers);
 
+	const std::string result = curlDataReceiver->get_data();
 	LOG(Level::DEBUG,
 		"OldReaderApi::get_subscribed_urls: document = %s",
 		result);
@@ -183,7 +177,7 @@ std::vector<TaggedFeedUrl> OldReaderApi::get_subscribed_urls()
 			auto url = strprintf::fmt("%s%s?n=%u",
 					OLDREADER_FEED_PREFIX,
 					id,
-					cfg->get_configvalue_as_int("oldreader-min-items"));
+					cfg.get_configvalue_as_int("oldreader-min-items"));
 			urls.push_back(TaggedFeedUrl(url, tags));
 		}
 	}
@@ -275,18 +269,19 @@ bool OldReaderApi::mark_article_read_with_token(const std::string& guid,
 std::string OldReaderApi::get_new_token()
 {
 	CurlHandle handle;
-	std::string result;
 	curl_slist* custom_headers{};
 
 	utils::set_common_curl_options(handle, cfg);
 	add_custom_headers(&custom_headers);
 	curl_easy_setopt(handle.ptr(), CURLOPT_HTTPHEADER, custom_headers);
-	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEFUNCTION, my_write_data);
-	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEDATA, &result);
 	curl_easy_setopt(handle.ptr(), CURLOPT_URL, OLDREADER_API_TOKEN_URL);
+
+	auto curlDataReceiver = CurlDataReceiver::register_data_handler(handle);
+
 	curl_easy_perform(handle.ptr());
 	curl_slist_free_all(custom_headers);
 
+	const std::string result = curlDataReceiver->get_data();
 	LOG(Level::DEBUG, "OldReaderApi::get_new_token: token = %s", result);
 
 	return result;
@@ -296,8 +291,8 @@ bool OldReaderApi::update_article_flags(const std::string& oldflags,
 	const std::string& newflags,
 	const std::string& guid)
 {
-	std::string star_flag = cfg->get_configvalue("oldreader-flag-star");
-	std::string share_flag = cfg->get_configvalue("oldreader-flag-share");
+	std::string star_flag = cfg.get_configvalue("oldreader-flag-star");
+	std::string share_flag = cfg.get_configvalue("oldreader-flag-share");
 	bool success = true;
 
 	if (star_flag.length() > 0) {
@@ -364,20 +359,21 @@ bool OldReaderApi::share_article(const std::string& guid, bool share)
 std::string OldReaderApi::post_content(const std::string& url,
 	const std::string& postdata)
 {
-	std::string result;
 	curl_slist* custom_headers{};
 
 	CurlHandle handle;
 	utils::set_common_curl_options(handle, cfg);
 	add_custom_headers(&custom_headers);
 	curl_easy_setopt(handle.ptr(), CURLOPT_HTTPHEADER, custom_headers);
-	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEFUNCTION, my_write_data);
-	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEDATA, &result);
 	curl_easy_setopt(handle.ptr(), CURLOPT_POSTFIELDS, postdata.c_str());
 	curl_easy_setopt(handle.ptr(), CURLOPT_URL, url.c_str());
+
+	auto curlDataReceiver = CurlDataReceiver::register_data_handler(handle);
+
 	curl_easy_perform(handle.ptr());
 	curl_slist_free_all(custom_headers);
 
+	const std::string result = curlDataReceiver->get_data();
 	LOG(Level::DEBUG,
 		"OldReaderApi::post_content: url = %s postdata = %s result = "
 		"%s",
