@@ -195,6 +195,7 @@ void RssParser::fill_feed_items(std::shared_ptr<RssFeed> feed,
 				upstream_feed.rss_version == rsspp::Feed::NEWSBLUR_JSON ||
 				upstream_feed.rss_version == rsspp::Feed::OCNEWS_JSON ||
 				upstream_feed.rss_version == rsspp::Feed::MINIFLUX_JSON ||
+				upstream_feed.rss_version == rsspp::Feed::FEEDBIN_JSON ||
 				upstream_feed.rss_version == rsspp::Feed::FRESHRSS_JSON) &&
 			item.labels.size() > 0) {
 			auto start = item.labels.begin();
@@ -249,6 +250,15 @@ void RssParser::fill_feed_items(std::shared_ptr<RssFeed> feed,
 				x->set_unread_nowrite(false);
 				x->set_override_unread(true);
 			}
+			if (std::find(start, finish, "feedbin:unread") !=
+				finish) {
+				x->set_unread_nowrite(true);
+				x->set_override_unread(true);
+			}
+			if (std::find(start, finish, "feedbin:read") != finish) {
+				x->set_unread_nowrite(false);
+				x->set_override_unread(true);
+			}
 		}
 
 		set_item_content(x, item);
@@ -292,9 +302,14 @@ void RssParser::set_item_title(std::shared_ptr<RssFeed> feed,
 	if (title.empty()) {
 		title = utils::make_title(item.link);
 		if (title.empty()) {
-			title = item.description;
-			x->set_title(render_xhtml_title(title, feed->link()));
-			return;
+			if (!item.description.empty()) {
+				x->set_title(render_xhtml_title(item.description, feed->link()));
+				return;
+			}
+			if (!item.content_encoded.empty()) {
+				x->set_title(render_xhtml_title(item.content_encoded, feed->link()));
+				return;
+			}
 		}
 	}
 
