@@ -23,6 +23,8 @@ extern "C" {
 #include <stfl.h>
 }
 
+#include "3rd-party/optional.hpp"
+
 #include "config.h"
 #include "colormanager.h"
 #include "controller.h"
@@ -47,6 +49,7 @@ extern "C" {
 #include "itemlistformaction.h"
 #include "itemview.h"
 #include "itemviewformaction.h"
+#include "keycombination.h"
 #include "keymap.h"
 #include "logger.h"
 #include "matcherexception.h"
@@ -118,7 +121,7 @@ void View::set_status(const std::string& msg)
 	auto fa = get_current_formaction();
 	if (fa != nullptr
 		&& std::dynamic_pointer_cast<EmptyFormAction>(fa) == nullptr) {
-		fa->set_value("msg", msg);
+		fa->set_status(msg);
 		fa->draw_form();
 	}
 }
@@ -142,7 +145,7 @@ bool View::run_commands(const std::vector<MacroCmd>& commands)
 		std::shared_ptr<FormAction> fa = get_current_formaction();
 		fa->prepare();
 		fa->draw_form();
-		if (!fa->process_op(command.op, true, &command.args)) {
+		if (!fa->process_op(command.op, command.args, BindingType::Macro)) {
 			// Operation failed, abort
 			return false;
 		}
@@ -155,7 +158,7 @@ int View::run()
 	bool have_macroprefix = false;
 
 	feedlist_form = std::make_shared<FeedListFormAction>(
-			this, feedlist_str, rsscache, filters, cfg, rxman);
+			*this, feedlist_str, rsscache, filters, cfg, rxman);
 	apply_colors(feedlist_form);
 	formaction_stack.push_back(feedlist_form);
 	current_formaction = formaction_stack_size() - 1;
@@ -214,15 +217,16 @@ int View::run()
 
 		LOG(Level::DEBUG, "View::run: event = %s", event);
 
+		const auto key_combination = KeyCombination::from_bindkey(event);
 		if (have_macroprefix) {
 			have_macroprefix = false;
 			status_line.show_message("");
 			LOG(Level::DEBUG,
 				"View::run: running macro `%s'",
 				event);
-			run_commands(keys->get_macro(event));
+			run_commands(keys->get_macro(key_combination));
 		} else {
-			const Operation op = keys->get_operation(event, fa->id());
+			const Operation op = keys->get_operation(key_combination, fa->id());
 
 			LOG(Level::DEBUG,
 				"View::run: event = %s op = %u",
@@ -236,7 +240,8 @@ int View::run()
 
 			// now we handle the operation to the
 			// formaction.
-			fa->process_op(op);
+			const std::vector<std::string> args;
+			fa->process_op(op, args);
 		}
 	}
 
@@ -283,7 +288,8 @@ std::string View::run_modal(std::shared_ptr<FormAction> f,
 			continue;
 		}
 
-		Operation op = keys->get_operation(event, fa->id());
+		const auto key_combination = KeyCombination::from_bindkey(event);
+		Operation op = keys->get_operation(key_combination, fa->id());
 
 		if (OP_REDRAW == op) {
 			Stfl::reset();
@@ -295,10 +301,11 @@ std::string View::run_modal(std::shared_ptr<FormAction> f,
 			continue;
 		}
 
-		fa->process_op(op);
+		const std::vector<std::string> args;
+		fa->process_op(op, args);
 	}
 
-	if (value == "") {
+	if (value.empty()) {
 		return "";
 	} else {
 		return f->get_value(value);
@@ -461,7 +468,7 @@ void View::push_searchresult(std::shared_ptr<RssFeed> feed,
 	if (feed->total_item_count() > 0) {
 		if (this->get_current_formaction()->id() != "searchresultslist") {
 			auto searchresult = std::make_shared<SearchResultsListFormAction>(
-					this, itemlist_str, rsscache, filters, cfg, rxman);
+					*this, itemlist_str, rsscache, filters, cfg, rxman);
 			apply_colors(searchresult);
 			searchresult->set_parent_formaction(get_current_formaction());
 			searchresult->add_to_history(feed, phrase);
@@ -491,7 +498,7 @@ std::shared_ptr<ItemListFormAction> View::push_itemlist(
 
 	if (feed->total_item_count() > 0) {
 		auto itemlist = std::make_shared<ItemListFormAction>(
-				this, itemlist_str, rsscache, filters, cfg, rxman);
+				*this, itemlist_str, rsscache, filters, cfg, rxman);
 		itemlist->set_feed(feed);
 		apply_colors(itemlist);
 		itemlist->set_parent_formaction(get_current_formaction());
@@ -530,7 +537,7 @@ void View::push_itemview(std::shared_ptr<RssFeed> f,
 			FormAction>(fa);
 		assert(itemlist != nullptr);
 		auto itemview = std::make_shared<ItemViewFormAction>(
-				this, itemlist, itemview_str, rsscache, cfg, rxman);
+				*this, itemlist, itemview_str, rsscache, cfg, rxman);
 		itemview->set_feed(f);
 		itemview->set_guid(guid);
 		itemview->set_parent_formaction(fa);
@@ -566,7 +573,7 @@ void View::view_dialogs()
 	auto fa = get_current_formaction();
 	if (fa != nullptr && fa->id() != "dialogs") {
 		auto dialogs = std::make_shared<DialogsFormAction>(
-				this, dialogs_str, cfg, rxman);
+				*this, dialogs_str, cfg, rxman);
 		dialogs->set_parent_formaction(fa);
 		apply_colors(dialogs);
 		dialogs->init();
@@ -580,7 +587,7 @@ void View::push_empty_formaction()
 	auto fa = get_current_formaction();
 
 	auto empty_view = std::make_shared<EmptyFormAction>(
-			this, empty_str, cfg);
+			*this, empty_str, cfg);
 	empty_view->set_parent_formaction(fa);
 	empty_view->init();
 	formaction_stack.push_back(empty_view);
@@ -592,7 +599,7 @@ void View::push_help()
 	auto fa = get_current_formaction();
 
 	auto helpview = std::make_shared<HelpFormAction>(
-			this, help_str, cfg, fa->id());
+			*this, help_str, cfg, fa->id());
 	apply_colors(helpview);
 	helpview->set_parent_formaction(fa);
 	helpview->init();
@@ -604,7 +611,7 @@ void View::push_urlview(const Links& links,
 	std::shared_ptr<RssFeed>& feed)
 {
 	auto urlview = std::make_shared<UrlViewFormAction>(
-			this, feed, urlview_str, cfg);
+			*this, feed, urlview_str, cfg);
 	apply_colors(urlview);
 	urlview->set_parent_formaction(get_current_formaction());
 	urlview->init();
@@ -613,23 +620,31 @@ void View::push_urlview(const Links& links,
 	current_formaction = formaction_stack_size() - 1;
 }
 
-std::string View::run_filebrowser(const std::string& default_filename)
+nonstd::optional<std::string> View::run_filebrowser(const std::string& default_filename)
 {
 	auto filebrowser = std::make_shared<FileBrowserFormAction>(
-			this, filebrowser_str, cfg);
+			*this, filebrowser_str, cfg);
 	apply_colors(filebrowser);
 	filebrowser->set_default_filename(default_filename);
 	filebrowser->set_parent_formaction(get_current_formaction());
-	return run_modal(filebrowser, "filenametext");
+	std::string res = run_modal(filebrowser, "filenametext");
+	if (res.empty()) {
+		return nonstd::nullopt;
+	}
+	return res;
 }
 
-std::string View::run_dirbrowser()
+nonstd::optional<std::string> View::run_dirbrowser()
 {
 	auto dirbrowser = std::make_shared<DirBrowserFormAction>(
-			this, filebrowser_str, cfg);
+			*this, filebrowser_str, cfg);
 	apply_colors(dirbrowser);
 	dirbrowser->set_parent_formaction(get_current_formaction());
-	return run_modal(dirbrowser, "filenametext");
+	std::string res = run_modal(dirbrowser, "filenametext");
+	if (res.empty()) {
+		return nonstd::nullopt;
+	}
+	return res;
 }
 
 std::string View::select_tag(const std::string& current_tag)
@@ -639,7 +654,7 @@ std::string View::select_tag(const std::string& current_tag)
 		return "";
 	}
 	auto selecttag = std::make_shared<SelectFormAction>(
-			this, selecttag_str, cfg);
+			*this, selecttag_str, cfg);
 	selecttag->set_type(SelectFormAction::SelectionType::TAG);
 	apply_colors(selecttag);
 	selecttag->set_parent_formaction(get_current_formaction());
@@ -652,7 +667,7 @@ std::string View::select_tag(const std::string& current_tag)
 std::string View::select_filter(const std::vector<FilterNameExprPair>& filters)
 {
 	auto selecttag = std::make_shared<SelectFormAction>(
-			this, selecttag_str, cfg);
+			*this, selecttag_str, cfg);
 	selecttag->set_type(SelectFormAction::SelectionType::FILTER);
 	apply_colors(selecttag);
 	selecttag->set_parent_formaction(get_current_formaction());
@@ -666,9 +681,9 @@ char View::confirm(const std::string& prompt, const std::string& charset)
 	LOG(Level::DEBUG, "View::confirm: charset = %s", charset);
 
 	std::shared_ptr<FormAction> f = get_current_formaction();
-	// Push empty formaction so our "msg" is not overwritten
+	// Push empty formaction so our status message is not overwritten on form `f`
 	push_empty_formaction();
-	f->set_value("msg", prompt);
+	f->set_status(prompt);
 
 	char result = 0;
 
@@ -692,7 +707,7 @@ char View::confirm(const std::string& prompt, const std::string& charset)
 			result);
 	} while (!result || strchr(charset.c_str(), result) == nullptr);
 
-	f->set_value("msg", "");
+	f->set_status("");
 	f->draw_form();
 
 	pop_current_formaction();
@@ -1038,7 +1053,7 @@ void View::pop_current_formaction()
 			std::shared_ptr<FormAction> fa = get_current_formaction();
 			if (fa) {
 				fa->set_redraw(true);
-				fa->set_value("msg", "");
+				f->set_status("");
 				fa->recalculate_widget_dimensions();
 			}
 		}
