@@ -1,5 +1,6 @@
 #include "cache.h"
 
+#include <memory>
 #include <sstream>
 
 #include "3rd-party/catch.hpp"
@@ -41,8 +42,8 @@ TEST_CASE("items in search result can be marked read", "[Cache]")
 TEST_CASE("Cleaning old articles works", "[Cache]")
 {
 	test_helpers::TempFile dbfile;
-	std::unique_ptr<ConfigContainer> cfg(new ConfigContainer());
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), cfg.get()));
+	auto cfg = std::make_unique<ConfigContainer>();
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 	const std::string uri = "file://data/rss.xml";
 	FeedRetriever feed_retriever(*cfg, *rsscache);
 	RssParser parser(uri, *rsscache, *cfg, nullptr);
@@ -70,9 +71,9 @@ TEST_CASE("Cleaning old articles works", "[Cache]")
 	 *
 	 * The value of 42 days is sufficient because the items in the test feed
 	 * are dating back to 2006. */
-	cfg.reset(new ConfigContainer());
+	cfg = std::make_unique<ConfigContainer>();
 	cfg->set_configvalue("keep-articles-days", "42");
-	rsscache.reset(new Cache(dbfile.get_path(), cfg.get()));
+	rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 	feed = rsscache->internalize_rssfeed("file://data/rss.xml", nullptr);
 
 	/* The important part: old articles should be gone, new one remains. */
@@ -81,9 +82,9 @@ TEST_CASE("Cleaning old articles works", "[Cache]")
 
 TEST_CASE("Last-Modified and ETag values are persisted to DB", "[Cache]")
 {
-	std::unique_ptr<ConfigContainer> cfg(new ConfigContainer());
+	auto cfg = std::make_unique<ConfigContainer>();
 	test_helpers::TempFile dbfile;
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), cfg.get()));
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 	const auto feedurl = "file://data/rss.xml";
 	FeedRetriever feed_retriever(*cfg, *rsscache);
 	RssParser parser(feedurl, *rsscache, *cfg, nullptr);
@@ -99,8 +100,8 @@ TEST_CASE("Last-Modified and ETag values are persisted to DB", "[Cache]")
 		REQUIRE_NOTHROW(rsscache->update_lastmodified(
 				feedurl, last_modified, etag));
 
-		cfg.reset(new ConfigContainer());
-		rsscache.reset(new Cache(dbfile.get_path(), cfg.get()));
+		cfg = std::make_unique<ConfigContainer>();
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 
 		/* Scrambling the value to make sure the following call changes
 		 * it. */
@@ -246,8 +247,8 @@ TEST_CASE(
 	};
 
 	std::vector<std::shared_ptr<RssFeed>> feeds;
-	std::unique_ptr<ConfigContainer> cfg(new ConfigContainer());
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), cfg.get()));
+	auto cfg = std::make_unique<ConfigContainer>();
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 	for (const auto& url : feedurls) {
 		FeedRetriever feed_retriever(*cfg, *rsscache);
 		RssParser parser(url, *rsscache, *cfg, nullptr);
@@ -260,8 +261,8 @@ TEST_CASE(
 		cfg->set_configvalue("cleanup-on-quit", "no");
 		rsscache->cleanup_cache(feeds);
 
-		cfg.reset(new ConfigContainer());
-		rsscache.reset(new Cache(dbfile.get_path(), cfg.get()));
+		cfg = std::make_unique<ConfigContainer>();
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 
 		for (const auto& url : feedurls) {
 			std::shared_ptr<RssFeed> feed =
@@ -279,8 +280,8 @@ TEST_CASE(
 			feeds.erase(feeds.cbegin(), feeds.cbegin() + 1);
 			rsscache->cleanup_cache(feeds);
 
-			cfg.reset(new ConfigContainer());
-			rsscache.reset(new Cache(dbfile.get_path(), cfg.get()));
+			cfg = std::make_unique<ConfigContainer>();
+			rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 
 			std::shared_ptr<RssFeed> feed =
 				rsscache->internalize_rssfeed(
@@ -300,8 +301,8 @@ TEST_CASE(
 
 			rsscache->cleanup_cache(feeds);
 
-			cfg.reset(new ConfigContainer());
-			rsscache.reset(new Cache(dbfile.get_path(), cfg.get()));
+			cfg = std::make_unique<ConfigContainer>();
+			rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 
 			std::shared_ptr<RssFeed> feed =
 				rsscache->internalize_rssfeed(
@@ -320,8 +321,8 @@ TEST_CASE(
 			feeds.erase(feeds.cbegin(), feeds.cbegin() + 1);
 			rsscache->cleanup_cache(feeds, always_clean);
 
-			cfg.reset(new ConfigContainer());
-			rsscache.reset(new Cache(dbfile.get_path(), cfg.get()));
+			cfg = std::make_unique<ConfigContainer>();
+			rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 
 			std::shared_ptr<RssFeed> feed =
 				rsscache->internalize_rssfeed(
@@ -341,8 +342,8 @@ TEST_CASE(
 
 			rsscache->cleanup_cache(feeds, always_clean);
 
-			cfg.reset(new ConfigContainer());
-			rsscache.reset(new Cache(dbfile.get_path(), cfg.get()));
+			cfg = std::make_unique<ConfigContainer>();
+			rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 
 			std::shared_ptr<RssFeed> feed =
 				rsscache->internalize_rssfeed(
@@ -379,13 +380,13 @@ TEST_CASE("get_read_item_guids returns GUIDs of items that are marked read",
 {
 	test_helpers::TempFile dbfile;
 	ConfigContainer cfg;
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), &cfg));
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 
 	// We'll keep our own count of which GUIDs are unread
 	std::unordered_set<std::string> read_guids;
 	const std::string uri1 = "file://data/rss.xml";
 	FeedRetriever feed_retriever(cfg, *rsscache);
-	std::unique_ptr<RssParser> parser(new RssParser(uri1, *rsscache, cfg, nullptr));
+	auto parser = std::make_unique<RssParser>(uri1, *rsscache, cfg, nullptr);
 	auto feed = parser->parse(feed_retriever.retrieve(uri1));
 
 	auto mark_read = [&read_guids](std::shared_ptr<RssItem> item) {
@@ -419,7 +420,7 @@ TEST_CASE("get_read_item_guids returns GUIDs of items that are marked read",
 	// Let's add another article to make sure get_unread_count looks at all
 	// feeds present in the Cache
 	const std::string uri2 = "file://data/atom10_1.xml";
-	parser.reset(new RssParser(uri2, *rsscache, cfg, nullptr));
+	parser = std::make_unique<RssParser>(uri2, *rsscache, cfg, nullptr);
 	feed = parser->parse(feed_retriever.retrieve(uri2));
 	mark_read(feed->items()[0]);
 	mark_read(feed->items()[2]);
@@ -430,7 +431,7 @@ TEST_CASE("get_read_item_guids returns GUIDs of items that are marked read",
 
 	// Lastly, let's make sure the info is indeed retrieved from the
 	// database and isn't just stored in the Cache object
-	rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+	rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 	INFO("Testing on two feeds with new `Cache` object");
 	check(rsscache->get_read_item_guids());
 }
@@ -440,7 +441,7 @@ TEST_CASE("mark_item_deleted changes \"deleted\" flag of item with given GUID ",
 {
 	test_helpers::TempFile dbfile;
 	ConfigContainer cfg;
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), &cfg));
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 	auto feedurl = "file://data/rss.xml";
 	FeedRetriever feed_retriever(cfg, *rsscache);
 	RssParser parser(feedurl, *rsscache, cfg, nullptr);
@@ -452,7 +453,7 @@ TEST_CASE("mark_item_deleted changes \"deleted\" flag of item with given GUID ",
 	rsscache->externalize_rssfeed(feed, false);
 	rsscache->mark_item_deleted(guid, true);
 
-	rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+	rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 	feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 	// One item was deleted, so shouldn't have been loaded
 	REQUIRE(feed->total_item_count() == 7);
@@ -463,7 +464,7 @@ TEST_CASE("mark_items_read_by_guid marks items with given GUIDs as unread ",
 {
 	test_helpers::TempFile dbfile;
 	ConfigContainer cfg;
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), &cfg));
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 	auto feedurl = "file://data/rss.xml";
 	FeedRetriever feed_retriever(cfg, *rsscache);
 	RssParser parser(feedurl, *rsscache, cfg, nullptr);
@@ -476,7 +477,7 @@ TEST_CASE("mark_items_read_by_guid marks items with given GUIDs as unread ",
 
 		REQUIRE_NOTHROW(rsscache->mark_items_read_by_guid({}));
 
-		rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 		feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 		REQUIRE(feed->unread_item_count() == 8);
 	}
@@ -489,7 +490,7 @@ TEST_CASE("mark_items_read_by_guid marks items with given GUIDs as unread ",
 
 		REQUIRE_NOTHROW(rsscache->mark_items_read_by_guid(guids));
 
-		rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 		feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 		REQUIRE(feed->unread_item_count() == 6);
 	}
@@ -590,7 +591,7 @@ TEST_CASE("update_rssitem_flags dumps `rss_item` object's flags to DB",
 {
 	test_helpers::TempFile dbfile;
 	ConfigContainer cfg;
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), &cfg));
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 	const auto feedurl = "file://data/rss.xml";
 	FeedRetriever feed_retriever(cfg, *rsscache);
 	RssParser parser(feedurl, *rsscache, cfg, nullptr);
@@ -601,7 +602,7 @@ TEST_CASE("update_rssitem_flags dumps `rss_item` object's flags to DB",
 	item->set_flags("abc");
 	REQUIRE_NOTHROW(rsscache->update_rssitem_flags(item.get()));
 
-	rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+	rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 	feed = rsscache->internalize_rssfeed("file://data/rss.xml", nullptr);
 
 	REQUIRE(feed->items()[0]->flags() == "abc");
@@ -614,7 +615,7 @@ TEST_CASE(
 {
 	test_helpers::TempFile dbfile;
 	ConfigContainer cfg;
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), &cfg));
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 	const auto feedurl = "file://data/rss.xml";
 	FeedRetriever feed_retriever(cfg, *rsscache);
 	RssParser parser(feedurl, *rsscache, cfg, nullptr);
@@ -630,7 +631,7 @@ TEST_CASE(
 
 		REQUIRE_NOTHROW(rsscache->update_rssitem_unread_and_enqueued(
 				item, feedurl));
-		rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 		feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 
 		REQUIRE_FALSE(feed->items()[0]->unread());
@@ -642,7 +643,7 @@ TEST_CASE(
 
 		REQUIRE_NOTHROW(rsscache->update_rssitem_unread_and_enqueued(
 				item, feedurl));
-		rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 		feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 
 		REQUIRE(feed->items()[0]->enqueued());
@@ -656,7 +657,7 @@ TEST_CASE(
 
 		REQUIRE_NOTHROW(rsscache->update_rssitem_unread_and_enqueued(
 				item, feedurl));
-		rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 		feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 
 		REQUIRE_FALSE(feed->items()[0]->unread());
@@ -714,7 +715,7 @@ TEST_CASE(
 
 	test_helpers::TempFile dbfile;
 	ConfigContainer cfg;
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), &cfg));
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 	const auto feedurl = "file://data/rss.xml";
 	FeedRetriever feed_retriever(cfg, *rsscache);
 	RssParser parser(feedurl, *rsscache, cfg, nullptr);
@@ -726,7 +727,7 @@ TEST_CASE(
 	SECTION("Simple case") {
 		rsscache->externalize_rssfeed(initial_feed, false);
 
-		rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 		auto new_feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 		new_feed->load();
 
@@ -738,7 +739,7 @@ TEST_CASE(
 		rsscache->externalize_rssfeed(initial_feed, false);
 		rsscache->externalize_rssfeed(initial_feed, false);
 
-		rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 		auto new_feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 		new_feed->load();
 
@@ -750,8 +751,8 @@ TEST_CASE("externalize_rssfeed doesn't store more than `max-items` items",
 	"[Cache]")
 {
 	test_helpers::TempFile dbfile;
-	std::unique_ptr<ConfigContainer> cfg(new ConfigContainer());
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), cfg.get()));
+	auto cfg = std::make_unique<ConfigContainer>();
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 
 	cfg->set_configvalue("max-items", "3");
 	const auto feedurl = "file://data/rss.xml";
@@ -761,8 +762,8 @@ TEST_CASE("externalize_rssfeed doesn't store more than `max-items` items",
 	REQUIRE(feed->total_item_count() == 8);
 	rsscache->externalize_rssfeed(feed, false);
 
-	cfg.reset(new ConfigContainer());
-	rsscache.reset(new Cache(dbfile.get_path(), cfg.get()));
+	cfg = std::make_unique<ConfigContainer>();
+	rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 	feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 	REQUIRE(feed->total_item_count() == 3);
 }
@@ -778,8 +779,8 @@ TEST_CASE(
 	"[Cache]")
 {
 	test_helpers::TempFile dbfile;
-	std::unique_ptr<ConfigContainer> cfg(new ConfigContainer());
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), cfg.get()));
+	auto cfg = std::make_unique<ConfigContainer>();
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 
 	const auto feedurl = "file://data/rss.xml";
 	FeedRetriever feed_retriever(*cfg, *rsscache);
@@ -790,9 +791,9 @@ TEST_CASE(
 	SECTION("no flagged items") {
 		rsscache->externalize_rssfeed(feed, false);
 
-		cfg.reset(new ConfigContainer());
+		cfg = std::make_unique<ConfigContainer>();
 		cfg->set_configvalue("max-items", "3");
-		rsscache.reset(new Cache(dbfile.get_path(), cfg.get()));
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 		feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 		REQUIRE(feed->total_item_count() == 3);
 	}
@@ -804,9 +805,9 @@ TEST_CASE(
 		rsscache->update_rssitem_flags(feed->items()[0].get());
 		rsscache->update_rssitem_flags(feed->items()[1].get());
 
-		cfg.reset(new ConfigContainer());
+		cfg = std::make_unique<ConfigContainer>();
 		cfg->set_configvalue("max-items", "1");
-		rsscache.reset(new Cache(dbfile.get_path(), cfg.get()));
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), cfg.get());
 		feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 		// All flagged items should be present no matter what
 		unsigned int flagged_count = 0;
@@ -861,7 +862,7 @@ TEST_CASE(
 {
 	test_helpers::TempFile dbfile;
 	ConfigContainer cfg;
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), &cfg));
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 	auto feedurl = "file://data/rss.xml";
 	FeedRetriever feed_retriever(cfg, *rsscache);
 	RssParser parser(feedurl, *rsscache, cfg, nullptr);
@@ -869,7 +870,7 @@ TEST_CASE(
 	feed->items()[0]->set_unread_nowrite(false);
 	rsscache->externalize_rssfeed(feed, false);
 
-	rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+	rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 	feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 	feed->load();
 	REQUIRE_FALSE(feed->items()[0]->unread());
@@ -878,14 +879,14 @@ TEST_CASE(
 
 	SECTION("reset_unread = false; item remains read") {
 		rsscache->externalize_rssfeed(feed, false);
-		rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 		feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 		REQUIRE_FALSE(feed->items()[0]->unread());
 	}
 
 	SECTION("reset_unread = true; item becomes unread") {
 		rsscache->externalize_rssfeed(feed, true);
-		rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 		feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 		REQUIRE(feed->items()[0]->unread());
 	}
@@ -898,7 +899,7 @@ TEST_CASE(
 {
 	test_helpers::TempFile dbfile;
 	ConfigContainer cfg;
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), &cfg));
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 	auto feedurl = "file://data/rss.xml";
 	FeedRetriever feed_retriever(cfg, *rsscache);
 	RssParser parser(feedurl, *rsscache, cfg, nullptr);
@@ -906,14 +907,14 @@ TEST_CASE(
 	feed->items()[0]->set_unread_nowrite(false);
 	rsscache->externalize_rssfeed(feed, false);
 
-	rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+	rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 	feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 	auto item = feed->items()[0];
 	item->set_unread_nowrite(true);
 
 	SECTION("override_unread not set; item remains read") {
 		rsscache->externalize_rssfeed(feed, false);
-		rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 		feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 		REQUIRE_FALSE(feed->items()[0]->unread());
 	}
@@ -921,7 +922,7 @@ TEST_CASE(
 	SECTION("override_unread is set; item becomes unread") {
 		item->set_override_unread(true);
 		rsscache->externalize_rssfeed(feed, false);
-		rsscache.reset(new Cache(dbfile.get_path(), &cfg));
+		rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 		feed = rsscache->internalize_rssfeed(feedurl, nullptr);
 		REQUIRE(feed->items()[0]->unread());
 	}
@@ -934,7 +935,7 @@ TEST_CASE(
 {
 	test_helpers::TempFile dbfile;
 	ConfigContainer cfg;
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), &cfg));
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 
 	auto feed = std::make_shared<RssFeed>(rsscache.get(),
 			"query:All unread:unread = \"yes\"");
@@ -955,7 +956,7 @@ TEST_CASE(
 	};
 	std::unique_ptr<sqlite3, sqlite_deleter> db;
 
-	// Open the database and manuever the pointer into our unique_ptr (which
+	// Open the database and maneuver the pointer into our unique_ptr (which
 	// can't be done by direct assignment because sqlite3_open doesn't
 	// return that pointer)
 	sqlite3* dbptr = nullptr;
@@ -990,7 +991,7 @@ TEST_CASE("do_vacuum doesn't throw an exception", "[Cache]")
 {
 	test_helpers::TempFile dbfile;
 	ConfigContainer cfg;
-	std::unique_ptr<Cache> rsscache(new Cache(dbfile.get_path(), &cfg));
+	auto rsscache = std::make_unique<Cache>(dbfile.get_path(), &cfg);
 	const std::string uri = "file://data/rss.xml";
 	FeedRetriever feed_retriever(cfg, *rsscache);
 	RssParser parser(uri, *rsscache, cfg, nullptr);
