@@ -1,6 +1,7 @@
 #include "itemlistformaction.h"
 
 #include <fstream>
+#include <tuple>
 #include <unistd.h>
 
 #include "3rd-party/catch.hpp"
@@ -57,10 +58,11 @@ TEST_CASE("OP_OPEN displays article using an external pager",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	ItemListFormAction itemlist(v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 
-	REQUIRE_NOTHROW(itemlist.process_op(OP_OPEN));
+	const std::vector<std::string> args;
+	REQUIRE_NOTHROW(itemlist.process_op(OP_OPEN, args));
 
 	test_helpers::assert_article_file_content(pagerfile.get_path(),
 		test_title,
@@ -87,17 +89,19 @@ TEST_CASE("OP_PURGE_DELETED purges previously deleted items",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	ItemListFormAction itemlist(v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 
 	SECTION("No items to purge") {
-		REQUIRE_NOTHROW(itemlist.process_op(OP_PURGE_DELETED));
+		const std::vector<std::string> args;
+		REQUIRE_NOTHROW(itemlist.process_op(OP_PURGE_DELETED, args));
 		REQUIRE(feed->total_item_count() == 1);
 	}
 
 	SECTION("Deleted items are purged") {
 		item->set_deleted(true);
-		REQUIRE_NOTHROW(itemlist.process_op(OP_PURGE_DELETED));
+		const std::vector<std::string> args;
+		REQUIRE_NOTHROW(itemlist.process_op(OP_PURGE_DELETED, args));
 		REQUIRE(feed->total_item_count() == 0);
 	}
 }
@@ -130,9 +134,10 @@ TEST_CASE(
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	ItemListFormAction itemlist(v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
-	itemlist.process_op(OP_OPENBROWSER_AND_MARK);
+	const std::vector<std::string> args;
+	itemlist.process_op(OP_OPENBROWSER_AND_MARK, args);
 	std::ifstream browserFileStream(browserfile.get_path());
 
 	REQUIRE(std::getline(browserFileStream, line));
@@ -167,9 +172,10 @@ TEST_CASE(
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	ItemListFormAction itemlist(v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
-	itemlist.process_op(OP_OPENBROWSER_AND_MARK);
+	const std::vector<std::string> args;
+	itemlist.process_op(OP_OPENBROWSER_AND_MARK, args);
 
 	REQUIRE(feed->unread_item_count() == 1);
 }
@@ -199,9 +205,10 @@ TEST_CASE("OP_OPENINBROWSER passes the url to the browser",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	ItemListFormAction itemlist(v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
-	itemlist.process_op(OP_OPENINBROWSER);
+	const std::vector<std::string> args;
+	itemlist.process_op(OP_OPENINBROWSER, args);
 	std::ifstream browserFileStream(browserfile.get_path());
 
 	REQUIRE(std::getline(browserFileStream, line));
@@ -233,9 +240,10 @@ TEST_CASE("OP_OPENINBROWSER_NONINTERACTIVE passes the url to the browser",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	ItemListFormAction itemlist(v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
-	itemlist.process_op(newsboat::OP_OPENINBROWSER_NONINTERACTIVE);
+	const std::vector<std::string> args;
+	itemlist.process_op(newsboat::OP_OPENINBROWSER_NONINTERACTIVE, args);
 	std::ifstream browserFileStream(browserfile.get_path());
 
 	REQUIRE(std::getline(browserFileStream, line));
@@ -275,7 +283,7 @@ TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	ItemListFormAction itemlist(v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 
 	SECTION("unread >= max-browser-tabs") {
@@ -284,7 +292,8 @@ TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 		cfg.set_configvalue(
 			"max-browser-tabs", std::to_string(maxItemsToOpen));
 
-		itemlist.process_op(OP_OPENALLUNREADINBROWSER);
+		const std::vector<std::string> args;
+		itemlist.process_op(OP_OPENALLUNREADINBROWSER, args);
 
 		std::ifstream browserFileStream(browserfile.get_path());
 		openedItemsCount = 0;
@@ -307,7 +316,8 @@ TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 		cfg.set_configvalue(
 			"max-browser-tabs", std::to_string(maxItemsToOpen));
 
-		itemlist.process_op(OP_OPENALLUNREADINBROWSER);
+		const std::vector<std::string> args;
+		itemlist.process_op(OP_OPENALLUNREADINBROWSER, args);
 
 		std::ifstream browserFileStream(browserfile.get_path());
 		if (browserFileStream.is_open()) {
@@ -359,7 +369,7 @@ TEST_CASE(
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	ItemListFormAction itemlist(v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 
 	SECTION("unread >= max-browser-tabs") {
@@ -368,7 +378,8 @@ TEST_CASE(
 		cfg.set_configvalue(
 			"max-browser-tabs", std::to_string(maxItemsToOpen));
 
-		itemlist.process_op(OP_OPENALLUNREADINBROWSER_AND_MARK);
+		const std::vector<std::string> args;
+		itemlist.process_op(OP_OPENALLUNREADINBROWSER_AND_MARK, args);
 
 		std::ifstream browserFileStream(browserfile.get_path());
 		if (browserFileStream.is_open()) {
@@ -392,7 +403,8 @@ TEST_CASE(
 		cfg.set_configvalue(
 			"max-browser-tabs", std::to_string(maxItemsToOpen));
 
-		itemlist.process_op(OP_OPENALLUNREADINBROWSER_AND_MARK);
+		const std::vector<std::string> args;
+		itemlist.process_op(OP_OPENALLUNREADINBROWSER_AND_MARK, args);
 
 		std::ifstream browserFileStream(browserfile.get_path());
 		if (browserFileStream.is_open()) {
@@ -443,7 +455,7 @@ TEST_CASE("OP_SHOWURLS shows the article's properties", "[ItemListFormAction]")
 	item->set_author(test_author);
 	item->set_description(test_description, "text/plain");
 	item->set_pubDate(test_pubDate);
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	ItemListFormAction itemlist(v, itemlist_str, &rsscache, filters, &cfg, rxman);
 
 	SECTION("with external-url-viewer") {
 		feed->add_item(item);
@@ -451,7 +463,8 @@ TEST_CASE("OP_SHOWURLS shows the article's properties", "[ItemListFormAction]")
 		cfg.set_configvalue(
 			"external-url-viewer", "tee > " + urlFile.get_path());
 
-		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS));
+		const std::vector<std::string> args;
+		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS, args));
 
 		test_helpers::assert_article_file_content(urlFile.get_path(),
 			test_title,
@@ -464,11 +477,13 @@ TEST_CASE("OP_SHOWURLS shows the article's properties", "[ItemListFormAction]")
 	SECTION("internal url viewer") {
 		feed->add_item(item);
 		itemlist.set_feed(feed);
-		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS));
+		const std::vector<std::string> args;
+		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS, args));
 	}
 
 	SECTION("no feed in formaction") {
-		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS));
+		const std::vector<std::string> args;
+		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS, args));
 	}
 }
 
@@ -502,7 +517,7 @@ TEST_CASE("OP_BOOKMARK pipes articles url and title to bookmark-command",
 	item->set_link(test_url);
 	item->set_title(test_title);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	ItemListFormAction itemlist(v, itemlist_str, &rsscache, filters, &cfg, rxman);
 
 	feed->add_item(item);
 	itemlist.set_feed(feed);
@@ -511,14 +526,28 @@ TEST_CASE("OP_BOOKMARK pipes articles url and title to bookmark-command",
 		"bookmark-cmd", "echo > " + bookmarkFile.get_path());
 
 	bookmark_args.push_back(extra_arg);
-	REQUIRE_NOTHROW(itemlist.process_op(OP_BOOKMARK, true, &bookmark_args));
 
-	std::ifstream browserFileStream(bookmarkFile.get_path());
+	auto checkOutput = [&] {
+		std::ifstream browserFileStream(bookmarkFile.get_path());
+
+		REQUIRE(std::getline(browserFileStream, line));
+		REQUIRE(line ==
+			test_url + separator + test_title + separator + extra_arg +
+			separator + feed_title);
+	};
+
+	SECTION("Macro") {
+		REQUIRE_NOTHROW(itemlist.process_op(OP_BOOKMARK, bookmark_args, BindingType::Macro));
+
+		checkOutput();
+	}
+
+	SECTION("Bind") {
+		REQUIRE_NOTHROW(itemlist.process_op(OP_BOOKMARK, bookmark_args, BindingType::Bind));
+
+		checkOutput();
+	}
 
-	REQUIRE(std::getline(browserFileStream, line));
-	REQUIRE(line ==
-		test_url + separator + test_title + separator + extra_arg +
-		separator + feed_title);
 }
 
 TEST_CASE("OP_EDITFLAGS arguments are added to an item's flags",
@@ -540,73 +569,42 @@ TEST_CASE("OP_EDITFLAGS arguments are added to an item's flags",
 	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	ItemListFormAction itemlist(v, itemlist_str, &rsscache, filters, &cfg, rxman);
 
 	feed->add_item(item);
 	itemlist.set_feed(feed);
 
-	SECTION("Single flag") {
-		std::string flags = "G";
-		op_args.push_back(flags);
+	// <name, input, expected_result>
+	std::vector<std::tuple<std::string, std::string, std::string>> tests {
+		std::make_tuple("Single flag", "G", "G"),
+		std::make_tuple("Unordered flags", "abdefc", "abcdef"),
+		std::make_tuple("Duplicate flag in argument", "Abdddd", "Abd"),
+		std::make_tuple("Unauthorized values in arguments: Numbers", "Abd1236", "Abd"),
+		std::make_tuple("Unauthorized values in arguments: Symbolds", "Abd%^\\*;\'\"&~#{([-|`_/@)]=}$£€µ,;:!?./§", "Abd"),
+		std::make_tuple("Unauthorized values in arguments: Accents", "Abd¨^", "Abd"),
+		std::make_tuple("All possible flags at once", "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz", "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"),
+	};
 
-		REQUIRE_NOTHROW(
-			itemlist.process_op(OP_EDITFLAGS, true, &op_args));
-		REQUIRE(item->flags() == flags);
-	}
-
-	SECTION("Unordered flags") {
-		std::string flags = "abdefc";
-		std::string ordered_flags = "abcdef";
-		op_args.push_back(flags);
-
-		REQUIRE_NOTHROW(
-			itemlist.process_op(OP_EDITFLAGS, true, &op_args));
-		REQUIRE(item->flags() == ordered_flags);
-	}
-
-	SECTION("Duplicate flag in argument") {
-		std::string flags = "Abd";
-		op_args.push_back(flags + "ddd");
-
-		REQUIRE_NOTHROW(
-			itemlist.process_op(OP_EDITFLAGS, true, &op_args));
-		REQUIRE(item->flags() == flags);
-	}
-
-	SECTION("Unauthorized values in arguments") {
-		std::string flags = "Abd";
-		SECTION("Numbers") {
-			op_args.push_back(flags + "1236");
-
-			REQUIRE_NOTHROW(itemlist.process_op(
-					OP_EDITFLAGS, true, &op_args));
-			REQUIRE(item->flags() == flags);
-		}
-		SECTION("Symbols") {
-			op_args.push_back(flags +
-				"%^\\*;\'\"&~#{([-|`_/@)]=}$£€µ,;:!?./§");
-
-			REQUIRE_NOTHROW(itemlist.process_op(
-					OP_EDITFLAGS, true, &op_args));
-			REQUIRE(item->flags() == flags);
-		}
-		SECTION("Accents") {
-			op_args.push_back(flags + "¨^");
-
-			REQUIRE_NOTHROW(itemlist.process_op(
-					OP_EDITFLAGS, true, &op_args));
-			REQUIRE(item->flags() == flags);
+	SECTION("Macro") {
+		for (const auto& testCase : tests) {
+			DYNAMIC_SECTION("Test " << std::get<0>(testCase)) {
+				std::vector<std::string> args = op_args;
+				args.push_back(std::get<1>(testCase));
+				REQUIRE_NOTHROW( itemlist.process_op(OP_EDITFLAGS, args, BindingType::Macro));
+				REQUIRE(item->flags() == std::get<2>(testCase));
+			}
 		}
 	}
 
-	SECTION("All possible flags at once") {
-		std::string flags =
-			"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
-		op_args.push_back(flags);
-
-		REQUIRE_NOTHROW(
-			itemlist.process_op(OP_EDITFLAGS, true, &op_args));
-		REQUIRE(item->flags() == flags);
+	SECTION("Bind") {
+		for (const auto& testCase : tests) {
+			DYNAMIC_SECTION("Test " << std::get<0>(testCase)) {
+				std::vector<std::string> args = op_args;
+				args.push_back(std::get<1>(testCase));
+				REQUIRE_NOTHROW( itemlist.process_op(OP_EDITFLAGS, args, BindingType::Bind));
+				REQUIRE(item->flags() == std::get<2>(testCase));
+			}
+		}
 	}
 }
 
@@ -648,19 +646,30 @@ TEST_CASE("OP_SAVE writes an article's attributes to the specified file",
 	item->set_pubDate(test_pubDate);
 	item->set_description(test_description, "text/plain");
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	ItemListFormAction itemlist(v, itemlist_str, &rsscache, filters, &cfg, rxman);
 
 	feed->add_item(item);
 	itemlist.set_feed(feed);
 
-	REQUIRE_NOTHROW(itemlist.process_op(OP_SAVE, true, &op_args));
+	auto check = [&] {
+		test_helpers::assert_article_file_content(saveFile.get_path(),
+			test_title,
+			test_author,
+			test_pubDate_str,
+			test_url,
+			test_description);
+	};
+
+	SECTION("Macro") {
+		REQUIRE_NOTHROW(itemlist.process_op(OP_SAVE, op_args, BindingType::Macro));
+		check();
+	}
+
+	SECTION("Bind") {
+		REQUIRE_NOTHROW(itemlist.process_op(OP_SAVE, op_args, BindingType::Bind));
+		check();
+	}
 
-	test_helpers::assert_article_file_content(saveFile.get_path(),
-		test_title,
-		test_author,
-		test_pubDate_str,
-		test_url,
-		test_description);
 }
 
 TEST_CASE("OP_HELP command is processed", "[ItemListFormAction]")
@@ -684,7 +693,8 @@ TEST_CASE("OP_HELP command is processed", "[ItemListFormAction]")
 
 	std::shared_ptr<ItemListFormAction> itemlist = v.push_itemlist(feed);
 
-	REQUIRE_NOTHROW(itemlist->process_op(OP_HELP));
+	const std::vector<std::string> args;
+	REQUIRE_NOTHROW(itemlist->process_op(OP_HELP, args));
 }
 
 TEST_CASE("OP_HARDQUIT command is processed", "[ItemListFormAction]")
@@ -705,10 +715,11 @@ TEST_CASE("OP_HARDQUIT command is processed", "[ItemListFormAction]")
 
 	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	ItemListFormAction itemlist(v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 
-	REQUIRE_NOTHROW(itemlist.process_op(OP_HARDQUIT));
+	const std::vector<std::string> args;
+	REQUIRE_NOTHROW(itemlist.process_op(OP_HARDQUIT, args));
 }
 
 TEST_CASE("Navigate back and forth using OP_NEXT and OP_PREV",
@@ -749,15 +760,16 @@ TEST_CASE("Navigate back and forth using OP_NEXT and OP_PREV",
 
 	std::shared_ptr<ItemListFormAction> itemlist = v.push_itemlist(feed);
 
-	REQUIRE_NOTHROW(itemlist->process_op(OP_NEXT));
-	itemlist->process_op(OP_SHOWURLS);
+	const std::vector<std::string> args;
+	REQUIRE_NOTHROW(itemlist->process_op(OP_NEXT, args));
+	itemlist->process_op(OP_SHOWURLS, args);
 
 	std::ifstream fileStream(articleFile.get_path());
 	std::getline(fileStream, line);
 	REQUIRE(line == prefix_title + second_article_title);
 
-	REQUIRE_NOTHROW(itemlist->process_op(OP_PREV));
-	itemlist->process_op(OP_SHOWURLS);
+	REQUIRE_NOTHROW(itemlist->process_op(OP_PREV, args));
+	itemlist->process_op(OP_SHOWURLS, args);
 
 	fileStream.seekg(0);
 	std::getline(fileStream, line);
@@ -788,13 +800,15 @@ TEST_CASE("OP_TOGGLESHOWREAD switches the value of show-read-articles",
 
 	SECTION("True to False") {
 		v.get_cfg()->set_configvalue("show-read-articles", "yes");
-		REQUIRE_NOTHROW(itemlist->process_op(OP_TOGGLESHOWREAD));
+		const std::vector<std::string> args;
+		REQUIRE_NOTHROW(itemlist->process_op(OP_TOGGLESHOWREAD, args));
 		REQUIRE_FALSE(v.get_cfg()->get_configvalue_as_bool(
 				"show-read-articles"));
 	}
 	SECTION("False to True") {
 		v.get_cfg()->set_configvalue("show-read-articles", "no");
-		REQUIRE_NOTHROW(itemlist->process_op(OP_TOGGLESHOWREAD));
+		const std::vector<std::string> args;
+		REQUIRE_NOTHROW(itemlist->process_op(OP_TOGGLESHOWREAD, args));
 		REQUIRE(v.get_cfg()->get_configvalue_as_bool(
 				"show-read-articles"));
 	}
@@ -838,19 +852,29 @@ TEST_CASE("OP_PIPE_TO pipes an article's content to an external command",
 	item->set_pubDate(test_pubDate);
 	item->set_description(test_description, "text/plain");
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	ItemListFormAction itemlist(v, itemlist_str, &rsscache, filters, &cfg, rxman);
 
 	feed->add_item(item);
 	itemlist.set_feed(feed);
 
-	REQUIRE_NOTHROW(itemlist.process_op(OP_PIPE_TO, true, &op_args));
+	auto check = [&] {
+		test_helpers::assert_article_file_content(articleFile.get_path(),
+			test_title,
+			test_author,
+			test_pubDate_str,
+			test_url,
+			test_description);
+	};
 
-	test_helpers::assert_article_file_content(articleFile.get_path(),
-		test_title,
-		test_author,
-		test_pubDate_str,
-		test_url,
-		test_description);
+	SECTION("Macro") {
+		REQUIRE_NOTHROW(itemlist.process_op(OP_PIPE_TO, op_args, BindingType::Macro));
+		check();
+	}
+
+	SECTION("Bind") {
+		REQUIRE_NOTHROW(itemlist.process_op(OP_PIPE_TO, op_args, BindingType::Bind));
+		check();
+	}
 }
 
 TEST_CASE("OP_OPENINBROWSER does not result in itemlist invalidation",
@@ -912,7 +936,8 @@ TEST_CASE("OP_OPENINBROWSER does not result in itemlist invalidation",
 		auto itemlist = v.push_itemlist(feed);
 		itemlist->prepare();
 
-		bool success = itemlist->process_op(OP_OPENINBROWSER);
+		const std::vector<std::string> args;
+		bool success = itemlist->process_op(OP_OPENINBROWSER, args);
 		Stfl::reset();
 		REQUIRE(success);
 
